#   ConVirt   -  Copyright (c) 2008 Convirture Corp.
#   ======
#
# ConVirt is a Virtualization management tool with a graphical user
# interface that allows for performing the standard set of VM operations
# (start, stop, pause, kill, shutdown, reboot, snapshot, etc...). It
# also attempts to simplify various aspects of VM lifecycle management.
#
#
# This software is subject to the GNU General Public License, Version 2 (GPLv2)
# and for details, please consult it at:
#
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
# author : Jd <jd_jedi@users.sourceforge.net>

import pylons
import simplejson as json
import traceback
from convirt.lib.base import BaseController
from tg import expose, flash, require, url, request, redirect,response,session,config
from convirt.model.CustomPredicates import authenticate
from convirt.model import *
from convirt.viewModel.DashboardService import DashboardService
from convirt.viewModel.ChartService import ChartService
from convirt.core.utils.utils import to_unicode,to_str
import convirt.core.utils.constants
constants = convirt.core.utils.constants
import logging,tg,os
LOGGER = logging.getLogger("convirt.controllers")

class DashboardController(BaseController):
    """

    """

    #allow_only = authenticate()
    #/dashboard/
    @expose('convirt.templates.login')
    def authenticate(self, came_from=url('/')):
        if session.get('userid') is None:
            try:
                redirect(url('/login', came_from=came_from, __logins=0))
            except Exception, e:
                #traceback.print_exc()
                raise Exception("SessionExpired.")

    @expose(template='json')
    def dashboardservice(self, type, node_id, username=None, password=None, ** kw):
        try:
            result = None
            self.authenticate()
            dashboardInfo = DashboardService().execute(session['auth'], type, node_id, username, password)
            result = json.dumps(dict(success=True,nodes=dashboardInfo.toJson()))
            return result
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def data_center_info(self, node_id, type):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().data_center_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def server_pool_info(self, node_id, type):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().server_pool_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def server_info(self, node_id, type):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().server_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def vm_info(self, node_id, type):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().vm_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def vm_availability(self,node_id,_dc=None):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().vm_availability(session['auth'], node_id)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def vm_storage(self,node_id,_dc=None):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().vm_storage(session['auth'], node_id)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))
    @expose(template='json')
    def dashboard_vm_info(self, node_id, type,_dc=None):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().dashboard_vm_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def dashboard_server_info(self, node_id, type,_dc=None):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().dashboard_server_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def dashboard_serverpool_info(self, node_id, type,_dc=None):
        self.authenticate()
        try:
            self.authenticate()
            info = DashboardService().dashboard_serverpool_info(session['auth'], node_id, type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))
    @expose(template='json')
    def metrics(self):
        try:
            self.authenticate()
            info = ChartService().metrics(session['auth'])
            #return info
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def get_chart_data(self,node_id,node_type,metric,period,offset,\
        frmdate=None,todate=None,chart_type=None,avg_fdate=None,avg_tdate=None):
        try:
            self.authenticate()
            info = ChartService().get_chart_data(session['auth'],\
                node_id,node_type,metric,period,offset,frmdate,todate,\
                chart_type,avg_fdate,avg_tdate)
            #return info
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def server_usage(self,node_id,metric,_dc=None):
        try:
            self.authenticate()
            info = DashboardService().server_usage(session['auth'],node_id,metric)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def topNvms(self,node_id,metric,node_type,_dc=None):
        try:
            self.authenticate()
            info = DashboardService().topNvms(session['auth'],node_id,metric,node_type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def topNservers(self,node_id,metric,node_type,_dc=None):
        try:
            self.authenticate()
            info = DashboardService().topNservers(session['auth'],node_id,metric,node_type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def os_distribution_chart(self,node_id,metric,node_type,_dc=None):
        try:
            self.authenticate()
            info = DashboardService().os_distribution_chart(session['auth'],node_id,metric,node_type)
            return dict(success=True,info=info)
        except Exception, ex:
            traceback.print_exc()
            return dict(success=False,msg=to_str(ex).replace("'", " "))

    @expose(template='json')
    def get_updated_tasks(self,user_name, _dc=None):
        try:
            result = None
            self.authenticate()
            result = DashboardService().get_updated_tasks(user_name)
            return dict(success="true",tasks=result)
        except Exception , ex:            
            return "{success:false,msg:'",to_str(ex).replace("'",""),"'}"

