# -*- coding: utf-8 -*-
#   ConVirt   -  Copyright (c) 2008 Convirture Corp.
#   ======
#
# ConVirt is a Virtualization management tool with a graphical user
# interface that allows for performing the standard set of VM operations
# (start, stop, pause, kill, shutdown, reboot, snapshot, etc...). It
# also attempts to simplify various aspects of VM lifecycle management.
#
#
# This software is subject to the GNU General Public License, Version 2 (GPLv2)
# and for details, please consult it at:
#
# http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
# author : Jd <jd_jedi@users.sourceforge.net>

import smtplib,traceback
import convirt.core.utils.utils
constants = convirt.core.utils.constants

from convirt.model import DBSession
from convirt.model.EmailSetup import EmailSetup
from email.MIMEText import MIMEText
from convirt.core.utils import ssmtplib
from convirt.core.utils.utils import to_str,to_unicode
import logging
LOGGER = logging.getLogger("convirt.model")

class EmailManager:

    def __init__(self):
        pass

    def send_email(self,email,message,subject="",mimetype=None):

        try:
            servers=DBSession.query(EmailSetup).all()
            if len(servers)==0:
                raise Exception("No EmailServers are setup. Can not send e-mail.")
            for server in servers:
                sent=self.send_mail( server, email, message, subject,mimetype)
                if sent:
                    return True

            return False
        except Exception, e:
            #traceback.print_exc()
            LOGGER.error("Error sending mails:"+to_str(e))
            print "Exception: ", e
            return False

    def send_mail(self,server,email,message,subject,mimetype='plain'):

        try:
            details=server.credential.cred_details
            server_user_name=details['user_email']
            server_user_psswd=details['password']

            success = False
            message = MIMEText(message, mimetype)
            message['Subject']= subject

            if (server.use_secure== constants.NONSECURE):
                success= self.send_nonsecure(server.mail_server,server.port,\
                            server_user_name,email,message.as_string())
            elif (server.use_secure== constants.TLS):
                success = self.send_tls(server.mail_server,server.port,\
                            server_user_name,server_user_psswd,email,message.as_string())
            else:
                success= self.send_ssl(server.mail_server,server.port,\
                            server_user_name,server_user_psswd,email,message.as_string())

            return success
        except Exception, e:
            #traceback.print_exc()
            LOGGER.error("Error sending mails:"+to_str(e))
            return False

    # to send email to non secure channel
    def send_nonsecure(self,mail_server,port,sender,receivers,msg):
        try:
            smtpObj = smtplib.SMTP(mail_server, port)
            smtpObj.sendmail(sender, receivers, msg)
            smtpObj.close()
            return True
        except Exception, e:
            #traceback.print_exc()
            LOGGER.error("Error sending mails:"+to_str(e))
            return False

    # to send email using TLS secure channel
    def send_tls(self,mail_server,port,sender,password,receivers,msg):
        try:
            server = smtplib.SMTP(mail_server, port)
            server.set_debuglevel(False)
            server.ehlo()
            server.starttls()
            server.ehlo()
            server.login(sender, password)
            server.sendmail(sender, receivers, msg)
            server.close()
            return True
        except Exception, e:
            #traceback.print_exc()
            LOGGER.error("Error sending mails:"+to_str(e))
            return False

    # to send email using SSL secure chanel
    def send_ssl(self,mail_server,port,sender,password,receivers,msg):
        try:
            server = ssmtplib.SMTP_SSL(mail_server, port)
            server.login(sender, password)
            try:
                server.sendmail(sender, receivers, msg)
            finally:
                server.close()
                return True
        except Exception, e:
            #traceback.print_exc()
            LOGGER.error("Error sending mails:"+to_str(e))
            return False


__author__="root"
__date__ ="$Jan 18, 2010 5:57:18 PM$"

if __name__ == "__main__":
    print "Hello";