#!/usr/bin/env python
#
#   ConVirt   -  Copyright (c) 2008 Convirture Corp.
#   ======
#
# ConVirt is a Virtualization management tool with a graphical user
# interface that allows for performing the standard set of VM operations
# (start, stop, pause, kill, shutdown, reboot, snapshot, etc...). It
# also attempts to simplify various aspects of VM lifecycle management.
#
#
# This software is subject to the GNU General Public License, Version 2 (GPLv2)
# and for details, please consult it at:
#
#    http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
#
#
#

from tg import session
from convirt.model.ManagedNode import ManagedNode
from convirt.model.Groups import ServerGroup
from convirt.core.utils.constants import *
import convirt.core.utils.utils
from convirt.core.utils.NodeProxy import Node
from pprint import pprint
from datetime import datetime
import time
from convirt.core.utils.utils import uuidToString,getHexID,dynamic_map,vm_config_write,\
                instantiate_configs,merge_pool_settings,validateVMSettings,randomUUID,copyToRemote,mktempfile
from convirt.core.utils.utils import to_unicode,to_str
utils = convirt.core.utils.utils
constants = convirt.core.utils.constants
from convirt.model.Entity import *
from convirt.model.DBHelper import DBHelper
import os, socket, tempfile, traceback, stat
from convirt.core.platforms.xen.XenNode import XenNode
from convirt.core.platforms.kvm.KVMNode import KVMNode
from convirt.model.Sites import Site
from convirt.model.ImageStore import ImageStore
from convirt.model.VM import VM, VMDisks, VMStorageLinks, VMStorageStat, VMDiskManager
from convirt.model.ImageStore import Image
from convirt.model import DBSession

from convirt.model.Authorization import AuthorizationService
from convirt.model.Metrics import MetricsService, MetricVMRaw, MetricVMCurr, MetricServerRaw, MetricServerCurr, DataCenterRaw, DataCenterCurr
from convirt.core.utils.phelper import AuthenticationException
from convirt.config.ConfigSettings import ClientConfiguration
from convirt.model.storage import StorageDef, StorageManager
from convirt.model.network import NwDef, NwManager
from convirt.model.SyncDefinition import SyncDef
from convirt.model.SPRelations import SPDefLink, DCDefLink, ServerDefLink, StorageDisks
from convirt.model.notification import Notification
from convirt.model.EmailManager import EmailManager

import logging,tg,time,transaction
LOGGER = logging.getLogger("convirt.model")

class GridManager:

    # creds_helper will eventually become part of XenNodeFactory
    def __init__(self,client_config, registry,creds_helper):
        self.client_config=client_config
        self.registry = registry
        self.group_list = {}
        self.node_list = {} # nodes that are not categorized
        self.storage_manager = StorageManager()
        self.network_manager = NwManager()
        self.sync_manager = SyncDef()
        
    def getDataCenters(self):
        dcs = DBHelper().get_all(Site)
        return dcs

    def getImageStores(self):
        iss = DBHelper().get_all(ImageStore)
        return iss

    def getFactory(self, platform):
        factory =  self.registry.get_node_factory(platform)
        if factory :
            return factory

        raise Exception("No factory for %s platform." % platform)

    def _create_default_groups(self):
        auth=AuthorizationService()
        #find the DataCenter Entity
        dc_id=getHexID(constants.DC,[constants.DATA_CENTER])
        dc=DBHelper().filterby(Entity,[],[Entity.entity_id==dc_id])[0]
        for name in ["Desktops", "QA Lab", "Servers"]:
            grp = ServerGroup(name)
            auth.add_entity(grp.name,grp.id,to_unicode(constants.SERVER_POOL),dc)
            DBHelper().add(grp)

        #self._save_groups()

    def _find_groups(self, node_name, group_list_map = None):
        # given a node name find all the group names to which it belongs
        grp = []

        if group_list_map is None:
            group_list_map = {}
            for g in self.group_list:
#                group_map[g] = self.group_list[g].getNodeNames()
                group_list_map[g] = {"node_list" : self.group_list[g].getNodeNames()}

        for group_name in group_list_map.keys():
            g = group_list_map[group_name]
            node_list = g["node_list"]
            if node_name in node_list:
                grp.append(group_name)

        return grp


    def discoverNodes(self,ip_list):
        pass

    def getGroupList(self,auth,site_id=None):
        if site_id:
            group_list=[]
            groups = DBSession.query(EntityRelation).filter_by(src_id = site_id)
            for eachgroup in groups:
                group = DBSession.query(ServerGroup).filter_by(id = eachgroup.dest_id).first()
                if group:
                    group_list.append(group)
            return group_list
        else:
            ents=auth.get_entities(to_unicode(constants.SERVER_POOL))
            ids = [ent.entity_id for ent in ents]
            grplist= DBHelper().filterby(ServerGroup,[],[ServerGroup.id.in_(ids)])
            return grplist



    def getGroupNames(self,auth):
        grpnames=auth.get_entity_names(to_unicode(constants.SERVER_POOL))
        return grpnames

    def getGroup(self,auth,groupId):
        ent=auth.get_entity(groupId)
        if ent is not None:
            grp=DBHelper().find_by_id(ServerGroup,groupId)
            return grp
        return None

    def get_dom(self, auth, domId):
        ent=auth.get_entity(domId)
        if ent is not None:
            return DBHelper().find_by_id(VM,domId)

    def get_doms(self,auth,nodeId):
        managed_node = self.getNode(auth,nodeId)
        if managed_node is None:
            raise Exception("Can not find the Server.")
        doms=[]
        node=auth.get_entity(nodeId)
        dom_names=auth.get_entity_names(to_unicode(constants.DOMAIN),parent= node)
        for domname in dom_names:
            single_dom = managed_node.get_dom(domname)
            if single_dom:
                doms.append(single_dom)
        return doms

    def get_running_doms(self,auth,nodeId):
        domlist=self.get_doms(auth,nodeId)
        runningdoms=[]
        for dom in domlist:
            if dom.is_resident():
                runningdoms.append(dom)
        return runningdoms

    def get_dom_names(self, auth, nodeId):
        ent=auth.get_entity(nodeId)
        if ent is not None:
            dom_names=auth.get_entity_names(to_unicode(constants.DOMAIN),parent=ent)
            return dom_names
        return []

    def getNodeNames(self, auth, groupId = None):
        if groupId is None:
            return []
        else:
            ent=auth.get_entity(groupId)
            nodes=auth.get_entity_names(to_unicode(constants.MANAGED_NODE),parent=ent)
            return nodes

    def getNodeList(self, auth, groupId=None):
        if groupId is None:
            return []
        else:
            ent=auth.get_entity(groupId)
            nodelist=[]
            if ent is not None:
                child_ents=auth.get_entities(to_unicode(constants.MANAGED_NODE),parent=ent)
                ids = [child_ent.entity_id for child_ent in child_ents]
                nodelist= DBHelper().filterby(ManagedNode,[],[ManagedNode.id.in_(ids)])
            return nodelist
        return []

    def getNode(self, auth, nodeId):
        ent=auth.get_entity(nodeId)
        if ent is not None:
            return DBHelper().find_by_id(ManagedNode,nodeId)


    def addNode(self,auth, node, groupId = None):
        if groupId is None:
            raise Exception("Invalid Group.")
        else:
            ent=auth.get_entity(groupId)
            if not auth.has_privilege('ADD_SERVER',ent):
                    raise Exception(constants.NO_PRIVILEGE)
            if ent is not None:
                try:
                    nodes=DBHelper().filterby(ManagedNode,[],[ManagedNode.hostname==node.hostname])
                    if len(nodes)>0:
                        raise Exception("Server %s already exists." % node.hostname)
                    auth.add_entity(node.hostname,node.id,to_unicode(constants.MANAGED_NODE),ent)
                    DBHelper().add(node)
                    
                    #call on_add_node() after adding the node to database since we should get reference of the node in the database while syncing and updating db table for status.
                    site = self.getSiteByGroupId(groupId)
                    site_id=None
                    if site:
                        site_id=site.id
                    self.sync_manager.on_add_node(node.id, groupId, site_id, auth, self.storage_manager)
                    self.sync_manager.on_add_node(node.id, groupId, site_id, auth, self.network_manager)                    
                except Exception, e:
                    raise e

    def removeNode(self,auth, nodeId):
        ent=auth.get_entity(nodeId)
        if not auth.has_privilege('REMOVE_SERVER',ent):
            raise Exception(constants.NO_PRIVILEGE)
        domlist=self.get_doms(auth,nodeId)

        #call on_remove_node() method before deleting entity and node since we should get node reference for storage/network def delete operation.
        groupId = ent.parents[0].entity_id
        site = self.getSiteByGroupId(groupId)
        site_id=None
        if site:
            site_id=site.id
        self.sync_manager.on_remove_node(nodeId, groupId, site_id, auth, self.storage_manager)
        self.sync_manager.on_remove_node(nodeId, groupId, site_id, auth, self.network_manager)

        for dom in domlist:
            self.remove_vm(auth, dom.id, nodeId) 
        
        auth.remove_entity_by_id(nodeId)
        # added code to delete record from curr metrics for the removed server
        MetricsService().DeleteCurrentMetrics(constants.SERVER_CURR,nodeId)
        node=DBHelper().find_by_id(ManagedNode,nodeId)
        DBHelper().delete(node)

    def editNode(self,auth,node):
        ent=auth.get_entity(node.id)
        if not auth.has_privilege('EDIT_SERVER',ent):
            raise Exception(constants.NO_PRIVILEGE)

        DBHelper().add(node)

    def transferNode(self, auth, source_group_id,dest_group_id, node_id, forcefully):
        vm_list=[]
        ent=auth.get_entity(node_id)
        grp=auth.get_entity(dest_group_id)
        if not auth.has_privilege('ADD_SERVER',grp) or not auth.has_privilege('TRANSFER_SERVER',ent):
            raise Exception(constants.NO_PRIVILEGE)
        if ent is not None:
            sync_manager = SyncDef()
            if forcefully == 'false':
                sync_manager.validate_transfer_node(node_id, source_group_id, auth)
            
            site = self.getSiteByGroupId(source_group_id)
            site_id=None
            if site:
                site_id = site.id
                
            self.sync_manager.on_transfer_node(node_id, source_group_id, dest_group_id, site_id, auth, self.storage_manager)
            self.sync_manager.on_transfer_node(node_id, source_group_id, dest_group_id, site_id, auth, self.network_manager)
            # here we are updating the entity irrespective of the status of the syncing operation. However we feel that there has to be a check whether all the definitions have been attached and detached properly before moving the entity. 
            auth.update_entity(ent,parent=grp)
            
            #Start - run matching algorithm
            vm_list = self.get_vms_from_pool(auth, source_group_id)
            if vm_list:
                for eachvm in vm_list:
                    self.remove_vm_storage_links_only(eachvm.id)
                    error = self.matching_on_AddEditDelete_vm(auth, 'TRANSFER_SERVER', eachvm.id)
                    if error:
                        LOGGER.error(to_str(error))
            #End - run matching algorithm


    def getNodeMetrics(self,auth,node):
        #managed_node=self.getNode(auth,node.hostname,group_name)
        #domnames=self.get_dom_names(auth, node.id)
        metrics=node.get_metrics()
        
        ###commented on 25/11/09
        ###node.get_metrics() returns only convirt related info
#        for key in metrics.keys():
#            if key not in domnames and key!='Domain-0' and isinstance(metrics[key],dict):
#                del metrics[key]
        ###end
        return metrics

    def refreshNodeMetrics(self,auth,node):
        try:
            node.connect()
        except AuthenticationException ,ex:
            raise Exception("Server not authenticated.")
        except Exception ,ex:            
            raise ex
        try:
            self.collectMetrics(auth,node)
        except Exception ,ex:            
            raise ex

    # Function to get the node metrics from the managed_node and insert
    # the metrics data in raw/current VM tables
    def collectMetrics(self, auth, managed_node):
        metrics=self.getNodeMetrics(auth,managed_node)
        ms = MetricsService()
        ent=auth.get_entity(managed_node.id)
        child_ents=auth.get_entities(to_unicode(constants.DOMAIN),parent=ent)
		# loop to get the vm id to be inserted in the VM metrics table since
        # the dictionary does not have VM id but VM name.
        for child_ent in child_ents:
            if child_ent.name not in metrics.keys():
                #ms.deleteMetricsData(child_ent.entity_id, 'VM_METRIC_CURR')
                continue
            else:
                #for keys in metrics:
                    #if keys==child_ent.name: # if the correct vm found
                        #dict_data = metrics[keys]
                        #vm_metrics_obj = MetricVMRaw()
                        #ms.insertMetricsData(dict_data, child_ent.entity_id,\
                                            #vm_metrics_obj, 'VM_METRIC_RAW')
                        #curr_metrics_obj = MetricVMCurr()
                        #ms.insertMetricsData(dict_data, child_ent.entity_id,\
                                        #curr_metrics_obj, 'VM_METRIC_CURR')
                continue

    # Function which gets the metrics data from the VM current metrics table
    # and returns a list of dictionaries to be displayed on the dashboard.
    def getCurrentMetrics(self, auth, managed_node):
        ms = MetricsService()
        vmmetrics = {}
        ent=auth.get_entity(managed_node.id)
        child_ents=auth.get_entities(to_unicode(constants.DOMAIN),parent=ent)
        for vm in child_ents:
            # data returned from the current metrics table.
            metrics = ms.getVMMetricsData('VM_METRIC_CURR', vm) 
            if metrics:
                vmmetrics[vm.name]=metrics
        return vmmetrics


    def do_node_action(self,auth,nodeId, action):
        ent=auth.get_entity(nodeId)
        if not auth.has_privilege(action.upper(),ent):
            raise Exception(constants.NO_PRIVILEGE)

        managed_node=self.getNode(auth,nodeId)
        domains = self.get_dom_names(auth,nodeId)
        print domains
        err_doms=[]
        for dom in domains:
            try:
                if action=='start_all':
                    if not managed_node.is_resident(dom) :
                        managed_node.start_dom(dom)
                elif action=='shutdown_all':
                    if managed_node.is_resident(dom) and managed_node.isDomU(dom):
                        managed_node.shutdown_dom(dom)
                elif action=='kill_all':
                    if managed_node.is_resident(dom) and managed_node.isDomU(dom):
                        d = managed_node.get_dom(dom)
                        if d is not None:
                            d._destroy()
            except Exception ,ex:
                traceback.print_exc()
                err_doms.append(dom+"-"+ to_str(ex))
        self.updateMetrics(auth,None,nodeId)
        if len(err_doms) > 0:
            err_str = "Error in VMs:- "+to_str(err_doms).replace("'", "")
            if len(err_doms) == len(domains):
                #all ops resulted in error, raise exception
                raise Exception(err_str)
            else:
                return err_str

    def restore_dom(self,auth, nodeId, file):
        ent=auth.get_entity(nodeId)
        if not auth.has_privilege('RESTORE_VM',ent):
            raise Exception(constants.NO_PRIVILEGE)
        managed_node=DBHelper().find_by_id(ManagedNode,nodeId)
        managed_node.restore_dom(file)

#    def list_nodes(self):
#        print "## DUMP =="
#        for name in self.getNodeNames():
#            print "Node name" ,  name
#        for g in self.group_list:
#            print "group ", g
#        print "## END DUMP =="



    def cloneNode(self,source_node, dest):
        pass


    def migrateDomains(self, auth, source_node, vm_list, dest, live, force=False, all=False):
        ex_list = []

        ent=auth.get_entity(dest.id)
        if not auth.has_privilege('ADD_VM',ent):
            raise Exception(constants.NO_PRIVILEGE)
        try:
            try:
                if len(vm_list) > 0 :
                    if not force:
                        (err_list, warn_list) = \
                                   source_node.migration_checks(vm_list,
                                                                dest, live)

                for vm in vm_list:
                    try:
                        ent=auth.get_entity(vm.id)
                        if ent is not None and not auth.has_privilege('MIGRATE_VM',ent):
                            raise Exception(constants.NO_PRIVILEGE)
                        #source_node.migrate_dom(vm.name, dest, live)
                        self.migrateDomain(auth, vm.name,source_node,dest, live,
                                           force = True) # checks done
                    except Exception, ex1:
                        traceback.print_exc()
                        ex_list.append("Error migrating " + vm.name + " : " + to_str(ex1))

            except Exception, ex:
                traceback.print_exc()
                raise ex
        finally:
            if len(ex_list) > 0:
                msg = "Errors in migrate all operations \n"
                for m in ex_list:
                    msg = msg + m + "\n"
                raise Exception(msg)



    def migrateNode(self, auth, source_node, dest, live, force = False):
        """ Migrate all vms on this node to a dest node."""

        ent1=auth.get_entity(source_node.id)
        ent2=auth.get_entity(dest.id)
        if not auth.has_privilege('MIGRATE_ALL',ent1) or not auth.has_privilege('ADD_VM',ent2):
            raise Exception(constants.NO_PRIVILEGE)

        vm_list = []
        for vm in self.get_doms(auth,source_node.id):
            if not vm.isDom0():
                vm_list.append(vm)

        self.migrateDomains(auth, source_node, vm_list, dest, live, force)

    def cloneDomain(self,source_dom_name,
                    source_node,
                    dest_node=None):
        pass

    def migrateDomain(self, auth, source_dom_name,
                      source_node,
                      dest_node, live, force = False):
        dom = source_node.get_dom(source_dom_name)
        running = dom.is_resident()

        if not force and running:
            (err_list, warn_list) = source_node.migration_checks([dom],
                                                                 dest_node,
                                                                 live)

        ## No good mechanism for sucess or failue till we cutover to
        # task / XenAPI
        try:
            if running:
                destid=dest_node.id
                dom.status=constants.MIGRATING                
                DBSession.add(dom)
                transaction.commit()
                dest_node=DBSession.query(ManagedNode).filter(ManagedNode.id==destid).one()
                source_node.migrate_dom(source_dom_name, dest_node, live)
        except socket.timeout:
            print "ignoring timeout on migration "
            pass
        except Exception, e:
            dom=DBSession.query(VM).filter(VM.id==dom.id).first()
            dom.status=None
            DBSession.add(dom)
            transaction.commit()
            raise e

        wait_time_over=False
        disappeared=False
        if running:
            wait_time=dom.get_wait_time('migrate')
            i=0
            while i <= wait_time:
                time.sleep(1)
                ###calling running vms: get metrics will filter out non convirt vms
                if disappeared==False and \
                        source_node.get_running_vms().has_key(source_dom_name):
                    if i==wait_time:
                        wait_time_over=True
                    i+=1
                    continue
                else:
                    disappeared=True
                    metrics=dest_node.get_running_vms()
                    if metrics.has_key(source_dom_name):
                        break
                    if i==wait_time:
                        wait_time_over=True
                    i+=1

        dom=DBSession.query(VM).filter(VM.id==dom.id).first()
        dom.status=None
        DBSession.add(dom)        

        if wait_time_over==True:
            transaction.commit()
            ###wait time exceeded. assume migrate failed.
            msg='VM did not appear in destination node within '\
                        +str(wait_time)+ 'seconds.'
            if disappeared==False:
                msg='VM still running in source node after '\
                        +str(wait_time)+ 'seconds.'
            raise Exception(msg)

        # move config files if necessary.
        #self.move_config_file(source_dom_name, source_node, dest_node)
        srvr=auth.get_entity(dest_node.id)
        auth.update_entity_by_id(dom.id,parent=srvr)
        transaction.commit()
        
    def move_config_file(self, dom_name, source_node, dest_node):
        dom = source_node.get_dom(dom_name)
        if dom and dom.get_config():
            config = dom.get_config()
            target_filename = config.filename
            isLink = False
            mode = source_node.node_proxy.lstat(target_filename).st_mode
            if stat.S_ISLNK(mode) is True:
                isLink = True
                print "CONFIG NAME  = ", config.filename
                target = source_node.node_proxy.readlink(config.filename)
                # transform the relative links to absolute
                print "ORIG TARGET  = ", target
                target = os.path.join(os.path.dirname(config.filename), target)
                print "TARGET  = ", target
                target_filename = os.path.abspath(target)
                print "TARGET FILENAME = ", target_filename


            if target_filename is not None:
                if dest_node.node_proxy.file_exists(target_filename):
                    # we are done:
                    pass
                else:
                    # create a temp file on the client node.
                    # and move it to the dest node.
                    (t_handle, t_name) = tempfile.mkstemp(prefix=dom_name)
                    try:
                        source_node.node_proxy.get(target_filename,
                                                   t_name)
                        utils.mkdir2(dest_node,
                                     os.path.dirname(target_filename))
                        dest_node.node_proxy.put(t_name,
                                                 target_filename)
                        source_node.node_proxy.remove(target_filename)

                    finally:
                        os.close(t_handle)
                        os.remove(t_name)

                if isLink:
                    dest_node.node_proxy.symlink(target_filename,
                                                 config.filename)
                    source_node.node_proxy.remove(config.filename)

            # now lets reassociate the config with the new node.
            dest_node.add_dom_config(config.filename)
            source_node.remove_dom_config(config.filename)



    # server pool related functions.

    def getGroupVars(self,auth, groupId):
        ent=auth.get_entity(groupId)
        if not auth.has_privilege('VIEW_GROUP_PROVISIONING_SETTINGS',ent):
            raise Exception(constants.NO_PRIVILEGE)
        group=DBHelper().find_by_id(ServerGroup,groupId)
        group_vars = group.getGroupVars()

        if not group_vars : # is None:
            # put some dummy ones for users to understand
            group_vars = {}
            group_vars["CLASS_A_STORAGE"] = "#/mnt/nfs_share/class_a"
            group_vars["CLASS_B_STORAGE"] = "#/mnt/nfs_share/class_b"
            group_vars["VM_DISKS_DIR"] = "#/mnt/shared/vm_disk"
            group_vars["VM_CONF_DIR"] = "#/mnt/shared/vm_configs"
            group_vars["DEFAULT_BRIDGE"] = "#br0"
        return group_vars

    def setGroupVars(self, auth, groupId,groupvars):
        ent=auth.get_entity(groupId)
        if not auth.has_privilege('EDIT_GROUP_PROVISIONING_SETTINGS',ent):
            raise Exception(constants.NO_PRIVILEGE)
        group=self.getGroup(auth,groupId)
        group.setGroupVars(groupvars)
        DBHelper().add(group)

    def addGroup(self,auth,grp,siteId):
        ent=auth.get_entity(siteId)
        if not auth.has_privilege('ADD_SERVER_POOL',ent):
            raise Exception(constants.NO_PRIVILEGE)
        try:
            grps=DBHelper().filterby(ServerGroup,[],[ServerGroup.name==grp.name])
            if len(grps)>0:
                raise Exception("Group %s already exists." % grp.name)
            auth.add_entity(grp.name,grp.id,to_unicode(constants.SERVER_POOL),ent)
            DBHelper().add(grp)
            
            self.sync_manager.on_add_group(grp.id)
            self.sync_manager.on_add_group(grp.id)
        except Exception, e:
            raise e

    def removeGroup(self,auth, groupId, deep=False):
        ent=auth.get_entity(groupId)
        if not auth.has_privilege('REMOVE_SERVER_POOL',ent):
            raise Exception(constants.NO_PRIVILEGE)

        site = self.getSiteByGroupId(groupId)
        site_id=None
        if site:
            site_id=site.id
        
        self.sync_manager.on_remove_group(site_id, groupId, auth, self.storage_manager)
        self.sync_manager.on_remove_group(site_id, groupId, auth, self.network_manager)

        if deep:
            for node in self.getNodeList(auth,groupId):
                self.removeNode(auth, node.id)
        auth.remove_entity(ent)
        grp=DBHelper().find_by_id(ServerGroup,groupId)
        DBHelper().delete(grp)
#        del self.group_list[groupId]
#        self._save_groups()


    def shutdown(self):
        # iterate through each node and disconnect it
        groups = self.getGroupList()
        for group in groups.itervalues():
            nodes = self.getNodeList(group.name)
            for n in nodes.itervalues():
                try:
                    n.disconnect()
                except Exception, ex:
                    print ex
        ungrouped_nodes = self.getNodeList()
        for n in ungrouped_nodes.itervalues():
            try:
                n.disconnect()
            except Exception, ex:
                print ex

   
    def import_dom_config(self, auth, nodeId,directory, file_list):
        ent=auth.get_entity(nodeId)
        if not auth.has_privilege("IMPORT_VM_CONFIG_FILE",ent):
            raise Exception(constants.NO_PRIVILEGE)

        managed_node = self.getNode(auth,nodeId)
        if managed_node is None:
            raise Exception("Can not find the Server.")

        for filename in file_list:
            try:
                file = os.path.join(directory, filename)
                dom = managed_node.add_dom_config(file)
                config_from_file = managed_node.new_config(file)
                config_from_file.update_storage_stats() # update storage stats
                config_from_file.write()
                doms=DBHelper().filterby(VM,[],[VM.name==dom.name])
                if len(doms)>0:
                    raise Exception("VM %s already exists." % dom.name)

                group_id = ent.parents[0].entity_id
                vm_disks = self.get_vm_disks_from_UI(dom.id, config_from_file)
                error =  self.pre_matching_on_AddEditDelete_vm(auth, "IMPORT_VM_CONFIG_FILE", dom.id, group_id, vm_disks)
                if error:
                    raise Exception(error)

                auth.add_entity(dom.name,dom.id,to_unicode(constants.DOMAIN),ent)
                DBHelper().add(dom)
                managed_node.refresh()
                self.updateMetrics(auth,None,nodeId)
                if dom:
                    new_config = dom.get_config()
                    self.import_config(auth, dom.id, nodeId, config_from_file)
            except Exception, ex:
                traceback.print_exc()
                err=to_str(ex).replace("'", " ")
                raise Exception('Error adding file, '+file+'. '+err)        

    def remove_vm(self,auth,domId,nodeId):
        ent=auth.get_entity(domId)
        if not auth.has_privilege('REMOVE_VM',ent):
            raise Exception(constants.NO_PRIVILEGE)
        dom = self.get_dom(auth,domId)
        if not dom:
            raise Exception("Can not find the specified VM.")
        managed_node = self.getNode(auth,nodeId)
        self.cleanupQCDomain(managed_node, dom)
        
        #remove all vm storage links and vm disks links.
        # This is because the VM is going to get deleted from the database
        self.remove_all_vm_storage_links(dom.id)
        
        DBHelper().delete(dom)
        auth.remove_entity_by_id(domId)
        ###added on 26/11/09
        MetricsService().DeleteCurrentMetrics(constants.VM_CURR, domId)
        self.updateMetrics(auth,domId,nodeId)
        ###end
        #managed_node.remove_dom_config(dom.get_config().filename)
        
        #matching vm disks with storage_disks and updating vm_storage_links table
        self.matching_on_AddEditDelete_vm(auth, "REMOVE_VM", dom.id)


    def remove_dom_config_file(self,auth,domId,nodeId):
        ent=auth.get_entity(domId)
        if not auth.has_privilege('REMOVE_VM_CONFIG',ent):
            raise Exception(constants.NO_PRIVILEGE)
        dom = self.get_dom(auth,domId)
        if not dom:
            raise Exception("Can not find the specified VM.")
#        filename=dom.get_config().filename
#        managed_node=self.getNode(auth,nodeId)
#        if managed_node.remove_dom_config(filename):
        
        #remove all vm storage links and vm disks links.
        # This is because the VM is going to get deleted from the database
        self.remove_all_vm_storage_links(dom.id)
        
        DBHelper().delete(dom)
        auth.remove_entity_by_id(domId)
        self.updateMetrics(auth,domId,nodeId)
        
    def save_dom_config_file(self,auth,domId,nodeId,content):
        ent=auth.get_entity(domId)
        if not auth.has_privilege('EDIT_VM_CONFIG_FILE',ent):
            raise Exception(constants.NO_PRIVILEGE)
        dom = self.get_dom(auth,domId)
        filename=dom.get_config().filename
        managed_node=self.getNode(auth,nodeId)
        file = managed_node.node_proxy.open(filename, "w")
        file.write(content)
        file.close()

    def save_dom(self,auth,domId,nodeId,file,directory):
        ent=auth.get_entity(domId)
        if not auth.has_privilege('SNAPSHOT_VM',ent):
            raise Exception(constants.NO_PRIVILEGE)
        managed_node = self.getNode(auth,nodeId)
        dom = managed_node.get_dom(domId)
        if directory and not managed_node.node_proxy.file_exists(directory):
            utils.mkdir2(managed_node, directory)
        if dom.is_resident():
            dom._save(file)

    def edit_vm_config(self, auth, vm_config, dom, context, group_id=None, vm_disks=None):
        try:
            ent=auth.get_entity(dom.id)
            if not auth.has_privilege('CHANGE_VM_SETTINGS',ent):
                raise Exception(constants.NO_PRIVILEGE)
            vm_config.set_managed_node(context.managed_node)
            vm_config.update_storage_stats()
            str_vm_config=""
            for name, value in vm_config.options.iteritems():
                str_vm_config+="%s = %s\n" % (name, repr(value))
                
            error = self.pre_matching_on_AddEditDelete_vm(auth, 'EDIT_VM_CONFIG', dom.id, group_id, vm_disks)
            if error:
                raise Exception(error)

            dom.vm_config=str_vm_config
            #dom.set_config=vm_config
            dom.template_version=context.template_version
            dom.os_flavor=vm_config["os_flavor"]
            dom.os_name=vm_config["os_name"]
            dom.os_version=vm_config["os_version"]
    
            #vm_config.write()
            DBHelper().add(dom)
        except Exception, ex:
            raise Exception(str(ex))
            
    def edit_image(self, auth, vm_config, image_config, context):
        
        ent=auth.get_entity(context.image.id)
        original_imageid=context.image.id


        if not auth.has_privilege('EDIT_IMAGE_SETTINGS',ent):
            raise Exception(constants.NO_PRIVILEGE)
#            vm_config.write()
#            image_config.write()
        if context.update_template:
            prev_image=context.image
            prev_image.id=getHexID()
#            print original_imageid,"==\n\n",prev_image.id

            DBHelper().add(prev_image)
            DBSession.flush()
            edit_image=Image(original_imageid, context.image.platform, context.image.name, context.image.location)
            edit_image.prev_version_imgid=original_imageid
#            edit_image.version=prev_image.version
            edit_image.version=context.new_version
#            edit_image.set_next_version()
        else:
            edit_image=context.image
#        print "----------------\n\n",edit_image.get_version()
        str_vm_config=""
        str_image_config=""

        for name, value in vm_config.options.iteritems():
            str_vm_config+="%s = %s\n" % (name, repr(value))
        for name, value in image_config.options.iteritems():
            str_image_config+="%s = %s\n" % (name, repr(value))

        edit_image.vm_config=str_vm_config
        edit_image.image_config=str_image_config

        edit_image.os_flavor=context.os_flavor
        edit_image.os_name=context.os_name
        edit_image.os_version=context.os_version

        DBHelper().add(edit_image)

    def edit_vm_info(self, auth, vm_config, vm_info, dom, context, \
                     initial_disks, final_disks, group_id=None, vm_disks=None):
        ent=auth.get_entity(dom.id)
        if not auth.has_privilege('CHANGE_VM_SETTINGS',ent):
            raise Exception(constants.NO_PRIVILEGE)
        vm_config.set_managed_node(context.managed_node)
        mem = vm_info["memory"]
        vcpus = vm_info["vcpus"]
        dom.setMem(mem)
        dom.setVCPUs(vcpus)
        
        error = self.pre_matching_on_AddEditDelete_vm(auth, 'EDIT_VM_INFO', dom.id, group_id, vm_disks)
        if error:
            raise Exception(error)
        
        detach_disk_list=[val for val in initial_disks \
                          if val not in final_disks]
        attach_disk_list=[val for val in final_disks \
                          if val not in initial_disks]
                          
        if detach_disk_list:
            dom.detachDisks(detach_disk_list)
        if attach_disk_list:
            dom.attachDisks(attach_disk_list)

        dom.template_version=context.template_version
        dom.os_flavor=vm_config["os_flavor"]
        dom.os_name=vm_config["os_name"]
        dom.os_version=vm_config["os_version"]

        vm_config.update_storage_stats()
        str_vm_config=""
        for name, value in vm_config.options.iteritems():
            str_vm_config+="%s = %s\n" % (name, repr(value))
        dom.vm_config=str_vm_config
        #dom.set_config=vm_config
        #vm_config.write()
        DBHelper().add(dom)

    # Provisioning process
    def provision(self,auth,context,img_name, group_id=None, vm_disks=None):

        # instantiate the configs
        v_config = context.vm_config
        i_config = context.image_config
        # add/set a uui to the config
        #v_config["uuid"] = uuidToString(randomUUID())

        # create is_remote map by position.
        image_id=context.image_id
        managed_node = context.managed_node
        start=context.start
        try:
            ent=auth.get_entity(managed_node.id)
            if not auth.has_privilege('PROVISION_VM',ent):
                raise Exception(constants.NO_PRIVILEGE)

            vmname=context.vm_config['name']
            doms=DBHelper().filterby(VM,[],[VM.name==vmname])
            if len(doms)>0:
                raise Exception("VM %s already exists." % vmname)
            id=getHexID()

            v_config["uuid"]=id
            
            store,managed_node,vm_config_file,vm_config=vm_config_write(auth,context,image_id,v_config,
                                                i_config,img_name)
            vm = managed_node.new_vm_from_config(vm_config)
            vm.id=id
            vm.image_id=image_id
            vm.template_version=context.image.version
            vm.os_flavor=v_config["os_flavor"]
            vm.os_name=v_config["os_name"]
            vm.os_version=v_config["os_version"]
            vm.created_user=auth.user.user_name
            vm.created_date=datetime.utcnow()
            
            error = self.pre_matching_on_AddEditDelete_vm(auth, 'PROVISION_VM', id, group_id, vm_disks)
            if error:
                raise Exception(error)

            (out, exit_code, log_filename) = \
                    store.execute_provisioning_script(auth,
                                                      managed_node,
                                                      image_id,
                                                      v_config,
                                                      i_config)
             # execute the provisioning script
            if exit_code == 0:
                try:
                    #managed_node.add_dom_config(vm_config_file)
                    auth.add_entity(vmname,id,to_unicode(constants.DOMAIN),ent)
                    vm_config.update_storage_stats()
                    str_vm_config=""
                    for name, value in vm_config.options.iteritems():
                        str_vm_config+="%s = %s\n" % (name, repr(value))
                    vm.vm_config=str_vm_config
                    DBHelper().add(vm)
                    transaction.commit()
                    if start=='yes':
                        self.do_dom_action(auth,id,managed_node.id,'start')
                    else:
                        self.updateMetrics(auth,id,managed_node.id)
                except Exception, e:
                    traceback.print_exc()
                    raise e
            else:
                print "out==",out, "exit_code=",exit_code,"log", log_filename
                raise Exception("Script Output\n------------------\n"+out+\
                                "\n------------------\n Log is available at "+log_filename)

        except Exception, e:
            traceback.print_exc()
            raise e

    def cleanupQCDomain(self,managed_node, dom, vbdlocation=''):
        """ Delete the xen configuration file and associated
        disk devices created during a quickcreate"""
        dom_config = None
        if dom:
            dom_config = dom.get_config()

        if dom_config:
            domfilename = dom_config.filename
            if domfilename and managed_node.node_proxy.file_exists(domfilename):
                managed_node.node_proxy.remove(domfilename)


            for file in dom_config.getDisks():
                # skip read only volumes, they are likely to be shared
                # or cdroms.
                # THIS IS NOT COMPLETE SOLN, NEED to prompt user for each
                # disk is the right soln
                if file.mode.find("w") == -1:
                    continue
                # dont mess with /dev/disk/ area. (Lun, AOE disks)
                if file.filename and (file.filename.strip().find("/dev/disk/")==0 or file.filename.strip().find("/dev/etherd/") == 0):
                    continue
                if file.type == 'phy':
                    try:
                        managed_node.lvm_proxy.removeLogicalVolume(file.filename)
                        print 'deleting: ' + file.filename
                    except OSError, err:
                        print "error deleting " + file.filename,err
                elif file.type in ['file','tap:aio','tap:qcow','tap:vmdk']:
                    if managed_node.node_proxy.file_exists(file.filename):
                        print 'deleting: ' + file.filename
                        managed_node.node_proxy.remove(file.filename)
        else:
            print "Couldn't find the domain %s. Skipping deletion" % name

    def do_dom_action(self,auth,domId,nodeId,action):
        ent=auth.get_entity(domId)
        if not auth.has_privilege(action.upper(),ent):
            raise Exception(constants.NO_PRIVILEGE)
        node=self.getNode(auth,nodeId)
        dom = node.get_dom(ent.name)
        if action=='start':
            if not dom.is_resident():
                dom._start()
                # added to update curr metrics immediaty to reflect the status in dashboard
                #commented because in kvm getmetrics pid becomes None
                #as we are setting the state of dom to VM.RUNNING
                #self.collectVMMetrics(auth, node)
        else:
            print " Else TEST_DASH", action, dom.is_resident()
            if dom.is_resident():
                if action=='pause':
                    dom._pause()                    
                if action=='unpause':
                    dom._unpause()                    
                if action=='reboot':
                    dom._reboot()
                if action=='shutdown':
                    dom._shutdown()
                    ###commented on 26/11/09
                    ###Collect metrics task will update the shutdown VMs status
                    #MetricsService().DeleteCurrentMetrics(constants.VM_CURR,domId)
                if action=='kill':
                    dom._destroy()
                    #MetricsService().DeleteCurrentMetrics(constants.VM_CURR,domId)
                    ###end

#        action_time = action + '_time'
#        config=dom.get_config()
#
#        sl=0.0
#        if config[action_time] == None:
#            sl=tg.config.get(eval('constants.'+action_time))
#            sl=int(sl)
#        else:
#            sl=config[action_time]

        sl = dom.get_wait_time(action)
        
        state=VM.SHUTDOWN    
        state_dict = {}
        state_dict['start']=[0,1]
        state_dict['shutdown']=[3]
        state_dict['kill']=[3]
        state_dict['pause']=[2]
        state_dict['unpause']=[0,1]
        state_dict['reboot']=[0,1] 
        values=state_dict[action]

        self.status_check(action,sl,state,values,node,dom)

        if action == 'start':
            metrics=node.get_metrics()
            data_dict=metrics.get(dom.name)
            if data_dict is not None:
                time.sleep(5)
            metrics=node.get_metrics()
            data_dict=metrics.get(dom.name)
            if data_dict is None:
                is_hvm=dom.is_hvm()
                #vm_info=dom.get_info()
                #dom_id=vm_info['domid']
                err=''
                if is_hvm == False:
                    f = open(constants.XEN_LOG_PATH, 'r')
                    err_log=self.tail(f,10)
                    for e in err_log:
                        err += e
                    raise Exception(err)

        node.refresh()
        self.updateMetrics(auth,domId,nodeId)

    def status_check(self,action,sl,state,values,node,dom):

        xl=0
        if action == 'reboot':
           rebooted=dom.check_reboot_state(sl)
           if rebooted:
               return
           else:
               raise Exception(action+' failed due to timeout')

        for counter in range(xl,sl):
            time.sleep(1)
            node.refresh()
            metrics=node.get_metrics()
            data_dict=metrics.get(dom.name)
                     
            if data_dict is not None:
                state=data_dict.get('STATE')
            if action=='shutdown' or action=='kill':
                if data_dict is None:
                    return
            elif state in values:
                return
            
        if action=='shutdown' or action=='kill':
           if data_dict is not None:
               raise Exception(action+' failed due to timeout')

        elif state not in values:
               raise Exception(action+' failed due to timeout')

    def tail(self,f, n):
        
        pos, lines = n+1, []
        while len(lines) <= n:
            try:
                try:
                    f.seek(-pos, 2)
                except IOError:
                  f.seek(0)
                  break
            finally:
                    lines = list(f)
            pos *= 2
        return lines[-n:]

    def set_dom_device(self,auth,dom,boot):
        ent=auth.get_entity(dom.id)
        if not auth.has_privilege('SET_BOOT_DEVICE',ent):
            raise Exception(constants.NO_PRIVILEGE)

        vm_config = dom.get_config()
        if not vm_config:
            raise Exception('No configuration file associated with this VM.')

        if boot is not None:
            vm_config['boot'] = boot
            str_vm_config=""
            for name, value in vm_config.options.iteritems():
                str_vm_config+="%s = %s\n" % (name, repr(value))
            dom.vm_config=str_vm_config
            DBHelper().add(dom)
            #vm_config.write()

    def save_appliance_info(self,auth,dom,config):
        ent=auth.get_entity(dom.id)
        if not auth.has_privilege('SAVE_APPLIANCE_INFO',ent):
            raise Exception(constants.NO_PRIVILEGE)

        str_vm_config=""
        for name, value in config.options.iteritems():
            str_vm_config+="%s = %s\n" % (name, repr(value))
        dom.vm_config=str_vm_config
        DBHelper().add(dom)
        #config.write()

    #Added by gizli: Cutover from NodeService
    def migrate_vm(self, auth, dom_list, source_node_id,\
                    dest_node_id, isLive, isForce, migrate_all): 
        managed_node = self.getNode(auth, source_node_id)
        dest_node = self.getNode(auth, dest_node_id)

#        vm_list = []
#        if migrate_all:
#            for vm in managed_node.get_doms():
#                if not vm.isDom0():
#                    vm_list.append(vm)
#        else:
#            dom = managed_node.get_dom(dom_name)
#            vm_list = [dom]
        managed_node.connect()
        dest_node.connect()
#        if not isForce:
#            (e, w) = managed_node.migration_checks(vm_list, dest_node, isLive)
#            if len(e) > 0 or len(w) > 0:
#                result = []
#                for err in e:
#                    (cat, msg) = err
#                    result.append(dict(type='error',category=cat,message=msg))
#                for warn in w:
#                    (cat, msg) = warn
#                    result.append(dict(type='warning',category=cat,message=msg))
#                return result
        vm_list=[]
        vm_name=[]
        metrics=managed_node.get_metrics()
        for domid in dom_list:
            doms=DBHelper().filterby(VM,[],[VM.id==domid])
            if len(doms)>0:                
                vm_list.append(doms[0])
                vm_state_dict=metrics.get(doms[0].name)
                if vm_state_dict is not None:
                    vm_name.append(doms[0].name)
        
        if migrate_all:
            self.migrateNode(auth, managed_node, dest_node, True, isForce)
        else:
            self.migrateDomains(auth, managed_node, vm_list, dest_node,\
                                True, isForce)
#        sl=0.0
#        sl=tg.config.get(constants.migrate_time)
#        sl=float(sl)
#        time.sleep(sl)

        self.updateMetrics(auth,None,dest_node.id)
        self.updateMetrics(auth,None,managed_node.id)

#        metrics=dest_node.get_metrics()
#        for v in vm_name:
#            data_dict=metrics.get(v)
#            if data_dict is None:
#               raise Exception('migrate failed')

    def get_vnc_info(self,auth,nodeId,domId,address):
        managed_node = self.getNode(auth,nodeId)
        dom = managed_node.get_dom(domId)
        result={}
        hostname=managed_node.address
        vnc_node=None
        if hostname is None:
            hostname=managed_node.hostname

        client_config=ClientConfiguration()
        vnchost=client_config.get(constants.prop_vnc_host)
        vncport=client_config.get(constants.prop_vnc_port)
        vncuser=client_config.get(constants.prop_vnc_user)
        vncpwd=client_config.get(constants.prop_vnc_password)
        
        if not vnchost or not vncport or not vncuser or vncport.find(':')==-1:
            raise Exception("VNC Host,Port & User should be configured properly.")
        
        (start,end)=vncport.split(':')
        if not start or not end:
            raise Exception("VNC Port should be configured properly."+\
                            "(start:end)")
#        forward_port=None
#        try:
#           forward_port=random.randrange(int(start),int(end))
#        except Exception, e:
#            print e
#            raise Exception("Port Numbers should be valid numbers.")
        
        result['hostname']=address
        result['port']='00'
        result['server']=managed_node.hostname
        result['vnc_display']='00'
        if not dom.is_resident():
            return result
        if dom.is_graphical_console():
            vnc_port = dom.get_vnc_port()
            #vnc_display = to_str(vnc_port)
            vnc_display = 5900+int(vnc_port)

#            list=DBHelper().filterby(ManagedNode,[],\
#                            [ManagedNode.hostname==vnchost])
            list=[]
            if len(list)>0:
                vnc_node=list[0]
            else:
                vnc_node=self.create_vnc_node(vnchost,vncuser,vncpwd)
            if vnc_node is None:
                raise Exception("Can not connect to VNC Host, "+vnchost+".")

            try:
#                 forward_port = self.get_vnc_forward_port(vnc_node,start,end)
                 forward_port = self.choose_vnc_display(int(start),int(end))
            except Exception, e:
                print "Exception: ", e
                raise Exception("Port Numbers should be valid numbers.")

            if forward_port is None:
                raise Exception("No ports are free in the given range."\
                                +"("+start+":"+end+")")
            cmd=self.get_port_forward_cmd(forward_port,vncuser,hostname,\
                                            vnc_display)
            print forward_port,vncuser,hostname,vnc_display
            print cmd
            print "------------------------------------------------------------"

            (output,exit_code)=vnc_node.node_proxy.exec_cmd(cmd)
            #print output,exit_code
            if exit_code==1:
                raise Exception("Error forwarding the vnc display to VNC Host.")

            if vnchost=='localhost':
                vnchost=address
            result['hostname']=vnchost
            result['port']=forward_port            
            result['server']=managed_node.hostname
            result['vnc_display']=vnc_display
        else:
            raise Exception('VNC is not enabled for this VM.')
        return result

    def create_vnc_node(self,hostname,username,password):
        use_keys=(password==None and hostname!='localhost')
        isRemote=(hostname!='localhost')
        vnc_node=ManagedNode(hostname = hostname,
                             ssh_port = 22,
                             username=username,
                             password=password,
                             isRemote=isRemote,
                             helper = None,
                             use_keys = use_keys,
                             address = hostname)
        try:
            vnc_node.connect()
        except Exception, e:
            print "Exception: ", e
            return None
        return vnc_node

    def get_vnc_forward_port(self,node,start,end):
        for i in range(int(start),int(end)):
            try:
                (output,exit_code)=node.node_proxy.exec_cmd("netstat -a | grep "+to_str(i))
                print "netstat-----",i,"---",output,"---",exit_code
                if output=='':
                    return i
            except Exception, e:
                print "Exception: ", e
                pass
        return None


    def choose_vnc_display(self,start,end):
        """Try to choose a free vnc display.
       """
        ports = self.netstat_local_ports()
        for port in range(start,end):
            if port in ports:
                continue
            return port
        return None



    def netstat_local_ports(self):
        """Run netstat to get a list of the local ports in use.
        """
        l = os.popen("netstat -nat").readlines()
        port_list = []
        # Skip 2 lines of header.
        for x in l[2:]:
            # Local port is field 3.
            y = x.split()[3]
            # Field is addr:port, split off the port.
            y = y.split(':')[-1]
            port_list.append(int(y))
#        print "ports===",port_list
        return port_list

    def get_port_forward_cmd(self,forward_port,vncuser,hostname,vnc_display):
        prename=to_str(forward_port)+"_"+to_str(vnc_display)+"_"
        temp_file =mktempfile(None,prename,".log")        
        cmd="socat -d -d -d -d TCP-LISTEN:"+to_str(forward_port)+\
            " EXEC:'/usr/bin/ssh "+vncuser+"@"+hostname+\
            " socat - TCP\:127.0.0.1\:"+to_str(vnc_display)+"' > "+temp_file+" 2>&1 &"
        return cmd

    """
    Function to get the node metrics from the managed_node and insert the metrics data in
    VM RAW/CURR tables
    """
    def collectVMMetrics(self, auth, managed_node, metrics=None):
        #print metrics,"\n\n\n"
        if metrics is None:
            metrics=self.getNodeMetrics(auth,managed_node)
        ms = MetricsService()
        vm_ents =[]
        ent=auth.get_entity(managed_node.id)
        child_ents=auth.get_entities(to_unicode(constants.DOMAIN),parent=ent)
        """
        loop to get the vm id to be inserted in the VM metrics table since the dictionary does not
        have VM id but VM name.
        """
        keys=metrics.keys()
        for child_ent in child_ents:
            dom=DBSession.query(VM).filter(VM.id==child_ent.entity_id).one()
            
            if dom.status==constants.MIGRATING:
                LOGGER.error("VM "+dom.name+", is on migration. "+\
                            "Not updating current metrics.")
                continue
            ###added on 26/11/09
            ###entering the shutdown vms status in metrics
            if child_ent.name in keys:                
                dict_data = metrics[child_ent.name]
                vm_metrics_obj = MetricVMRaw()
                ms.insertMetricsData(dict_data, child_ent.entity_id, vm_metrics_obj)
            else:
                dict_data={'STATE':VM.SHUTDOWN}
                ###to show storage related info for shutdown vms
                ###metrics will have info only gor running vms
                managed_node.augment_storage_stats(child_ent.name, dict_data, dom)
                vm_metrics_obj = MetricVMRaw()
                ms.insertMetricsData(dict_data, child_ent.entity_id, vm_metrics_obj)                
#            for keys in metrics:
#                if keys==child_ent.name: # if the correct vm found
#                    dict_data = metrics[keys]
#                    vm_metrics_obj = MetricVMRaw()
#                    ms.insertMetricsData(dict_data, child_ent.entity_id, vm_metrics_obj)
            ###end

    """
    Function to get the node metrics from the managed_node and insert the metrics data in
    SERVER RAW/CURR tables
    """ 
    def collectServerMetrics(self, auth, m_node):
        # commneting newly added code to update server metrics
        if not m_node.is_authenticated():
            try:
                m_node.connect()
            except Exception ,ex:
                #traceback.print_exc()
                LOGGER.error(ex)
        if m_node.is_authenticated():
            node_status = "Connected"
        else:
            node_status = "Not Connected"

        node_snapshot={}
        if m_node is not None:                
            try:
                if m_node.is_authenticated() and not m_node.is_in_error():
                    node_snapshot = None
                    #dom_count = 0
                    try:
                        node_snapshot = self.getNodeMetrics(auth,m_node)
                    except Exception ,ex:
                        print "error getting info for ", m_node.hostname, ex
                        traceback.print_exc()
                        pass

                    if node_snapshot is None:
                        node_snapshot = {}
                        node_snapshot["NODE_NAME"] = m_node.hostname

                    node_snapshot["NODE_NAME"]= m_node.hostname
                    node_snapshot["NODE_STATUS"]= node_status
                    ###commented on 25/11/09
                    ###node.get_metrics() returns total vms
    #                try:
    #                    dom_count = m_node.get_VM_count()
    #                    node_snapshot["TOTAL_VMs"] = dom_count
    #                except Exception ,ex:
    #                    #print "error getting dom count ", m_node.hostname, ex
    #                    pass
                    ###end
                else:
                    node_snapshot = {"NODE_NAME":m_node.hostname,
                                        "NODE_STATUS":node_status
                                        }
            except Exception, e:
                traceback.print_exc()
                LOGGER.error(e)

            node_snapshot["NODE_PLATFORM"] = m_node.platform

        ms = MetricsService()
        server_metrics_obj = MetricServerRaw()
        ms.insertServerMetricsData(node_snapshot, m_node.id, server_metrics_obj)
        return node_snapshot

    """
    Function to insert the metrics data in SERVER_POOL RAW/CURR tables. the data in the SERVER_POOL
    is rolled-up data from server metrics table
    """    
    def collectServerPoolMetrics(self, auth, pool_id):
        node_list = self.getNodeList(auth, pool_id)
        connected = 0
        # obtain the server connected status to display in the dashboard        
        for m_node in node_list:
            if m_node is None :            
                continue
            if m_node is not None:
                try:
                    if not m_node.is_authenticated():
                        m_node.connect()
                    if m_node.is_authenticated() and not m_node.is_in_error():
                        connected = connected + 1
                except Exception, e:
                    LOGGER.error(e)
        """
        added to insert serverpool metrics. the calculation for pool is
        done from the values of server_metrics_raw table. the function
        takes pool_id as a parameter.
        """
        ms = MetricsService()
        ms.collect_serverpool_metrics(pool_id,connected,auth)

    """
    This is a thread function to collect metrics at VM/SERVER/POOL level at the specified
    interval.This function is responsible for updating current as well as raw tables.
    """
    
    def collect_metrics_for_all_nodes(self, auth):
        serverpool_ents = auth.get_entities(to_unicode(constants.SERVER_POOL))
        # loop through all the server_pools present in the data centre
        for serverpool_ids in serverpool_ents:
            node_list = self.getNodeList(auth, serverpool_ids.entity_id)
            #node_list.sort()
            for m_node in node_list:
                
                if m_node is None :
                    continue 

                #call function to store the Server metrics into the database
                node_snapshot=self.collectServerMetrics(auth, m_node)

                #call function to store the VM metrics into the database table
                self.collectVMMetrics(auth, m_node, node_snapshot) 

            #collect metrics at serverpool level
            self.collectServerPoolMetrics(auth, serverpool_ids.entity_id)
            DBSession.flush()
            transaction.commit()

    def updateMetrics(self, auth, domId=None,nodeId=None,groupId=None):
        if domId is None and nodeId is None and groupId is None:
            self.collect_metrics_for_all_nodes(auth)
            return

        if groupId is not None:
            node_list = self.getNodeList(auth, groupId)
            for m_node in node_list:
                if m_node is None :
                    continue

                #print "\n\nGROUPPPPP\n\n"
                node_snapshot=self.collectServerMetrics(auth, m_node)
                self.collectVMMetrics(auth, m_node, node_snapshot)

            self.collectServerPoolMetrics(auth, groupId)
            return

        if nodeId is not None:
            ent=auth.get_entity(nodeId)
            m_node=self.getNode(auth, nodeId)

            if m_node is None :
                return

            node_snapshot=self.collectServerMetrics(auth, m_node)
            self.collectVMMetrics(auth, m_node, node_snapshot)
            #print "\n\nNODEEEEEEEEEEEEE\n\n"
            pool_id=ent.parents[0].entity_id
            self.collectServerPoolMetrics(auth, pool_id)
            return 

        if domId is not None:
            ent=auth.get_entity(domId)
            node=ent.parents[0]
            m_node=self.getNode(auth, node.entity_id)

            if m_node is None :
                return

            node_snapshot=self.collectServerMetrics(auth, m_node)
            self.collectVMMetrics(auth, m_node, node_snapshot)
            #print "\n\nDOMMMMMMMMMMMMM\n\n"
            pool_id=node.parents[0].entity_id
            self.collectServerPoolMetrics(auth, pool_id)
            return 

    def getSiteByGroupId(self, group_id):
        site=None
        entity = DBSession.query(EntityRelation).filter_by(dest_id = group_id).first()
        if entity:
            site = DBSession.query(Site).filter_by(id = entity.src_id).first()
        return site

    def getSite(self, site_id):
        site = DBSession.query(Site).filter_by(id=site_id).first()
        return site

    def send_notifications(self,auth):
        """
        send E-mail for failed tasks.
        """
        notifications=DBSession.query(Notification).filter_by(mail_status = False).all()
        emanager=EmailManager()
        for n in notifications:
            sent=False
            email = n.emailId
            message = n.task_name + " Task failed at "+ to_str(n.error_time) \
                        + "\n\n" +to_str(n.error_msg)
            subject = "ConVirt - Failed Task: "+n.task_name
            sent=emanager.send_email(email, message, subject, 'plain')
            if sent == True:
                n.mail_status=True
                DBHelper().update(n)
            else:
                LOGGER.error("Error Sending Notification:-"+subject) 

    def get_vm_disks_from_UI(self, vm_name, config):
        vm_disks=[]
        if config:
            vm_id=None
            vm = DBSession.query(VM).filter_by(name=vm_name).first()
            if vm:
                vm_id = vm.id

            storage_status_object = config.get("storage_status_object")
            if storage_status_object:
                disk_stat = storage_status_object.get("disk_stat")
                if disk_stat:
                    for each_disk_stat in disk_stat:
                        disk={}
                        vm_disk_id=None
                        objVMDisk = DBSession.query(VMDisks).filter_by(vm_id=vm_id, disk_name=each_disk_stat.get("filename")).first()
                        if objVMDisk:
                            vm_disk_id = objVMDisk.id
                        disk["vm_id"] = vm_id
                        disk["id"] = vm_disk_id
                        disk["disk_name"] = each_disk_stat.get("filename")
                        disk["read_write"] = each_disk_stat.get("mode")
                        vm_disks.append(disk)
        return vm_disks

    
    def manage_vm_disks(self, auth, vm_name, node_id, config, mode):
        if config:
            vm = DBSession.query(VM).filter_by(name=vm_name).first()
            vm_config=None
            if vm:
                vm_id = vm.id
                vm_config = vm.get_config()
            else:
                vm_id = None
            
            storage_status_object = config.get("storage_status_object")
            if storage_status_object:
                disk_stat = storage_status_object.get("disk_stat")
                if disk_stat:
                    if len(disk_stat)>0:
                        #remove all vm storage links
                        self.remove_all_vm_storage_links(vm_id)

                    for each_disk_stat in disk_stat:
                        #each_disk_stat.get("type")
                        storage_disk_id = each_disk_stat.get("storage_disk_id")
                        disk_name = each_disk_stat.get("filename")
                        dev_type = each_disk_stat.get("device")
                        read_write = each_disk_stat.get("mode")
                        is_shared = each_disk_stat.get("shared")
                        actual_size = 0
                        if disk_name:
                            actual_size,disk_dev_type = VMDiskManager(vm_config).get_disk_size(None, disk_name)
                        
                        actual_size_GB = self.storage_manager.convert_to_GB_from_Bytes(actual_size)
                        disk_size = each_disk_stat.get("size")
                        disk_size_GB = self.storage_manager.convert_to_GB(disk_size)
                        if mode == "EDIT_VM_CONFIG" or mode == "EDIT_VM_INFO":
                            if disk_size=="null" or disk_size=="" or disk_size==None or disk_size==0:
                                disk_size = self.storage_manager.convert_to_MB(actual_size_GB)
                                disk_size_GB = self.storage_manager.convert_to_GB(disk_size)
                            else:
                                disk_size_GB = each_disk_stat.get("size")
                        
                        disk_type = each_disk_stat.get("type")
                        file_system = each_disk_stat.get("fs_type")
                        storage_id = each_disk_stat.get("storage_id")
                            
                        #add in storage_disks for NFS while provisioning. Since while provisioning it creates disk.
                        if mode == "PROVISION_VM":
                            #here add allocated as false in matching algorith, it will be set appropriatly.
                            storage_allocated=False
                            returnVal = self.storage_manager.add_storage_disk(storage_id, actual_size_GB, disk_size_GB, disk_name, None, None, None, None, storage_allocated, self)
                            if returnVal:
                                storage_disk_id = returnVal

                        vm_disk = self.add_vm_disk(vm_id, disk_name, disk_size_GB, dev_type, read_write, disk_type, is_shared, file_system)
        
        #matching vm disks with storage_disks and updating vm_storage_links table
        error_msg=None
        error_msg = self.matching_on_AddEditDelete_vm(auth, mode, vm_id)
        if error_msg:
            raise Exception(error_msg)

    
    def add_vm_disk(self, vm_id, disk_name, disk_size, dev_type, read_write, disk_type, is_shared, file_system, vm_memory=None):
        #add in vm_disks table
        vm_disk = VMDisks()
        vm_disk.id = getHexID()
        vm_disk.vm_id = vm_id
        vm_disk.disk_name = disk_name
        if not disk_size:
            disk_size=0
        vm_disk.disk_size = float(disk_size)
        vm_disk.dev_type = dev_type
        vm_disk.read_write = read_write
        vm_disk.disk_type = disk_type
        vm_disk.is_shared = is_shared
        vm_disk.file_system = file_system
        if vm_memory:
            vm_disk.vm_memory = vm_memory
        else:
            vm_disk.vm_memory = self.get_vm_memory(vm_id)
        DBSession.add(vm_disk)
        return vm_disk
        
    def add_vm_storage_link(self, vm_disk_id, storage_disk_id):
        try:
            #when we run the matching algorith for the rest of the vms except the vm which is getting edited,
            #we find the links in the add_vm_storage_link table. So in this case we should not add duplicate link.
            #And we should not throw unique contraint exception also to keep the operation go on. So this check is here.
            vm_link = DBSession.query(VMStorageLinks).filter_by(vm_disk_id=vm_disk_id, storage_disk_id=storage_disk_id).first()
            if not vm_link:
                #Add in vm_storage_links table. Establish vm disk and storage disk link here.
                if vm_disk_id and storage_disk_id:
                    vm_storage_link = VMStorageLinks()
                    vm_storage_link.id = getHexID()
                    vm_storage_link.vm_disk_id = vm_disk_id
                    vm_storage_link.storage_disk_id = storage_disk_id
                    DBSession.add(vm_storage_link)
        except Exception, ex:
            LOGGER.error("Can not add duplicate vm disk and storage disk link.")
        
    def get_vm_memory(self, vm_id):
        vm_memory=0
        vm = DBSession.query(VM).filter_by(id=vm_id).first()
        if vm:
            vm_config = vm.vm_config
            if vm_config:
                vm_config_param_list = str(vm_config).split("=")
                for each_param in vm_config_param_list:
                    if each_param[0].strip() == "memory":
                        vm_memory = each_param[1].strip()
                        
        return vm_memory

    def remove_vm_storage_link(self, vm_disk_id):
        vm_storage_link = DBSession.query(VMStorageLinks).filter_by(vm_disk_id=vm_disk_id).first()
        if vm_storage_link:
            #Remove vm storage link here. Remove from vm_storage_links table.
            storage_disk_id = vm_storage_link.storage_disk_id
            DBSession.delete(vm_storage_link)
            
            #calculate disk size
            vm_disk = DBSession.query(VMDisks).filter_by(id=vm_disk_id).first()
            if vm_disk:
                disk_size = vm_disk.disk_size
            op = "-"
            self.storage_manager.calculate_disk_size(storage_disk_id, vm_disk.id, disk_size, op)

    def remove_vm_storage_links_only(self, vm_id):
        vm_disks = DBSession.query(VMDisks).filter_by(vm_id=vm_id)
        if vm_disks:
            for vm_disk in vm_disks:
                #Remove the vm storage link from vm_storage_links table.
                self.remove_vm_storage_link(vm_disk.id)
        
    def remove_all_vm_storage_links(self, vm_id):
        vm_disks = DBSession.query(VMDisks).filter_by(vm_id=vm_id)
        if vm_disks:
            for vm_disk in vm_disks:
                #Remove the vm storage link from vm_storage_links table.
                self.remove_vm_storage_link(vm_disk.id)
                #Remove the vm disk from vm_disks table.
                self.remove_vm_disk(vm_disk.id)

    def remove_vm_links_to_storage(self, storage_id):
        storage_disks = DBSession.query(StorageDisks).filter_by(storage_id=storage_id)
        if storage_disks:
            for eachdisk in storage_disks:
                vm_storage_links = DBSession.query(VMStorageLinks).filter_by(storage_disk_id=eachdisk.id)
                if vm_storage_links:
                    for eachlink in vm_storage_links:
                        #Remove the vm storage link from vm_storage_links table.
                        self.remove_vm_storage_link(eachlink.vm_disk_id)

        
    def remove_vm_disk(self, vm_disk_id):
        vm_disk = DBSession.query(VMDisks).filter_by(id=vm_disk_id).first()
        if vm_disk:
            DBSession.delete(vm_disk)
            
    
    def import_config(self, auth, vm_id, node_id, config):
        storage_disk_id=None
        disk_name=None
        disk_size=0
        dev_type=None
        read_write=None
        storage_type=None   #file/physical device
        is_shared=False  #shared/unshared
        file_system=None
        vm_space=None
        
        if config:
            storage_stats = config.get("STORAGE_STATS")
            disks = config.get("disk")
            vm_space = config.get("memory")
            if storage_stats:
                disk_stats = storage_stats.get("DISK_STATS")
                if disk_stats:
                    i=0
                    for each_disk_stat in disk_stats:
                        if type(each_disk_stat) == str:
                            objDiskStat = disk_stats.get(each_disk_stat)
                            if objDiskStat:
                                #dev_type = objDiskStat.get("DEV_TYPE")
                                disk_name = objDiskStat.get("DISK_NAME")
                                disk_size = objDiskStat.get("DISK_SIZE")
                        
                        if disks:
                            each_disk = disks[i]
                            each_disk_param = each_disk.split(",")
                            file_name_type = str(each_disk_param[0]).split(":")
                            if file_name_type:
                                storage_type = file_name_type[0]
                                file_name = file_name_type[1]
                            dev_type = each_disk_param[1]
                            read_write = each_disk_param[2]

                        #add in vm_disks table
                        if not storage_disk_id:
                            storage_disk_id = node_id
                        disk_size_temp = self.storage_manager.convert_to_GB_from_Bytes(disk_size)
                        vm_disk = self.add_vm_disk(vm_id, disk_name, disk_size_temp, dev_type, read_write, storage_type, is_shared, file_system)
                        i += 1
        #matching vm disks with storage_disks and updating vm_storage_links table
        self.matching_vm_disks(auth, vm_id)

    def pre_matching_on_AddEditDelete_vm(self, auth, mode, vm_id, group_id, vm_disks=None):
        #This function is called on provision, edit setting and remove vm.
        #The vm disk which is become free, check whether any other vm is using the disk and establish a link between vm and storage disk for the vm.
        error_msg=None
        if mode == "PROVISION_VM" or mode == "IMPORT_VM_CONFIG_FILE" or mode == "TRANSFER_SERVER":
            error_msg = self.pre_matching_vm_disks(auth, vm_id, group_id, vm_disks)
        elif mode == "EDIT_VM_CONFIG" or mode == "EDIT_VM_INFO" or mode == "REMOVE_VM":
            #run the matching logic first for the vm which is getting edited.
            error_msg = self.pre_matching_vm_disks(auth, vm_id, group_id, vm_disks)
            #run the matching logic for rest of the vm.
            #get all the vm
            vms = DBSession.query(VM)
            if vms:
                for vm in vms:
                    if vm.id != vm_id:
                        #do not return error message for rest of the vms so it should not throw the exception and operation should keep going on.
                        self.pre_matching_vm_disks(auth, vm.id, group_id)
        return error_msg

    def matching_on_AddEditDelete_vm(self, auth, mode, vm_id):
        #This function is called on provision, edit setting and remove vm.
        #The vm disk which is become free, check whether any other vm is using the disk and establish a link between vm and storage disk for the vm.
        error_msg=None
        if mode == "PROVISION_VM" or mode == "TRANSFER_SERVER":
            error_msg = self.matching_vm_disks(auth, vm_id)
        elif mode == "EDIT_VM_CONFIG" or mode == "EDIT_VM_INFO" or mode == "REMOVE_VM":
            #run the matching logic first for the vm which is getting edited.
            error_msg = self.matching_vm_disks(auth, vm_id)
            #run the matching logic for rest of the vm.
            #get all the vm
            vms = DBSession.query(VM)
            if vms:
                for vm in vms:
                    if vm.id != vm_id:
                        #do not return error message for rest of the vms so it should not throw the exception and operation should keep going on.
                        self.matching_vm_disks(auth, vm.id)
        return error_msg
                
    def get_vms_from_pool(self, auth, group_id):
        vm_list=[]
        if group_id:
            group_entity = auth.get_entity(group_id)
            node_entities = auth.get_entities(to_unicode(constants.MANAGED_NODE), group_entity)
            if node_entities:
                for eachnode in node_entities:
                    vm_entities = auth.get_entities(to_unicode(constants.DOMAIN), eachnode)
                    for eachvm in vm_entities:
                        vm = DBSession.query(VM).filter_by(id=eachvm.entity_id).first()
                        if vm:
                            vm_list.append(vm)
        return vm_list
                
    def get_vm_disks_from_pool(self, auth, group_id):
        vm_disk_list=[]
        if group_id:
            group_entity = auth.get_entity(group_id)
            node_entities = auth.get_entities(to_unicode(constants.MANAGED_NODE), group_entity)
            if node_entities:
                for eachnode in node_entities:
                    vm_entities = auth.get_entities(to_unicode(constants.DOMAIN), eachnode)
                    for eachvm in vm_entities:
                        vm_disks = DBSession.query(VMDisks).filter_by(vm_id=eachvm.entity_id)
                        if vm_disks:
                            for eachdisk in vm_disks:
                                vm_disk_list.append(eachdisk)
        return vm_disk_list
        
    def get_storage_disks_from_pool(self, group_id):
        storage_disk_list=[]
        if group_id:
            sp_def_links = DBSession.query(SPDefLink).filter_by(group_id=group_id)
            if sp_def_links:
                for eachlink in sp_def_links:
                    defn = self.storage_manager.get_defn(eachlink.def_id)
                    if defn:
                        storage_disks = DBSession.query(StorageDisks).filter_by(storage_id=defn.id)
                        if storage_disks:
                            for eachdisk in storage_disks:
                                storage_disk_list.append(eachdisk)
        return storage_disk_list
                                        
    def get_storage_disks_from_storage(self, storage_id):
        storage_disks = DBSession.query(StorageDisks).filter_by(storage_id=storage_id)
        return storage_disks

    def pre_matching_vm_disks(self, auth, vm_id, group_id, vm_disks=None):
        error_msg = None
        storage_disks = None
        if group_id:
            storage_disks = self.get_storage_disks_from_pool(group_id)

        if vm_id:
            #get all the vm disks related to the vm.
            if not vm_disks:
                vm_disks = DBSession.query(VMDisks).filter_by(vm_id=vm_id)
            if vm_disks:
                if storage_disks:
                    #compare vm disk with storage disk on unique_path.
                    for each_vm_disk in vm_disks:
                        for each_storage_disk in storage_disks:
                            msg = None
                            isMatched, msg = self.pre_matching_logic(each_vm_disk, each_storage_disk)
                            if msg:
                                error_msg = msg
                            if isMatched==True:
                                break
        return error_msg

                        
    def matching_vm_disks(self, auth, vm_id):
        error_msg = None
        if vm_id:
            storage_disks=[]
            vm_entity = auth.get_entity(vm_id)
            if vm_entity:
                node_entity = vm_entity.parents[0]
                group_entity = node_entity.parents[0]
                if group_entity:
                    storage_disks = self.get_storage_disks_from_pool(group_entity.entity_id)
                    
            #get all the vm disks related to the vm.
            vm_disks = DBSession.query(VMDisks).filter_by(vm_id=vm_id)
            if vm_disks:
                if storage_disks:
                    #compare vm disk with storage disk on unique_path.
                    for each_vm_disk in vm_disks:
                        for each_storage_disk in storage_disks:
                            msg = None
                            isMatched, msg = self.matching_logic(each_vm_disk, each_storage_disk)
                            if msg:
                                error_msg = msg
                            if isMatched==True:
                                break
        return error_msg
                            

    def matching_disk_on_discover_storage(self, vm_disks, storage_disk_id):
        #This would be executed when you would associate any storage disk to the server pool.
        #It will try to match that disk to any existing vm disk.
        #if it is matched then it would create a link in vm_storage_links table.
        #get all the vm disks related to all the vms.
        if vm_disks:
            #get the newly added storage disk
            storage_disk = DBSession.query(StorageDisks).filter_by(id=storage_disk_id).first()
            if storage_disk:
                #compare vm disk with storage disk on unique_path.
                for each_vm_disk in vm_disks:
                    self.matching_logic(each_vm_disk, storage_disk)

    def pre_matching_logic(self, vm_disk, storage_disk):
        isMatched=False
        msg = None
        try:
            vm_disk_id = vm_disk.id
            disk_name = vm_disk.disk_name
            read_write = vm_disk.read_write
        except Exception, ex:
            vm_disk_id = vm_disk.get("id")
            disk_name = vm_disk.get("disk_name")
            read_write = vm_disk.get("read_write")

        if to_str(disk_name).strip() == to_str(storage_disk.unique_path).strip():
            #matching disk is found here
            LOGGER.info("Matching disk (" + str(disk_name) + ") is found")
            #check whether storage is allocated or not. If it is not then
            #update vm_disk table with storage_disk_id
            if storage_disk.storage_allocated == False:
                #storage is not allocated
                LOGGER.info("Storage (" + str(disk_name) + ") is not allocated")
                isMatched=True
            elif read_write == "r":
                #storage is allocated
                LOGGER.info("Storage (" + to_str(disk_name) + ") is allocated. The disk is in readonly mode.")
                isMatched=True
            else:
                isMatched=True
                vm_link = DBSession.query(VMStorageLinks).filter_by(vm_disk_id=vm_disk_id, storage_disk_id=storage_disk.id).first()
                if vm_link:
                    LOGGER.info("vm disk and storage disk link already exists so we are not throwing exception or logging error.")
                else:
                    LOGGER.error("Matching disk (" + str(disk_name) + ") is found. The disk is already allocated and in read-write mode. So it is invalid disk entry.")
                    msg = "Invalid disk entry."
        return (isMatched, msg)
            
    def matching_logic(self, vm_disk, storage_disk):
        isMatched=False
        msg = None
        if str(vm_disk.disk_name).strip() == str(storage_disk.unique_path).strip():
            #matching disk is found here
            LOGGER.info("Matching disk (" + str(vm_disk.disk_name) + ") is found")
            #check whether storage is allocated or not. If it is not then
            #update vm_disk table with storage_disk_id
            if storage_disk.storage_allocated == False:
                #storage is not allocated
                LOGGER.info("Storage (" + str(vm_disk.disk_name) + ") is not allocated")
                #Add link in vm_storage_links table.
                self.add_vm_storage_link(vm_disk.id, storage_disk.id)
                #update the shared and read_write mode
                vm_disk.is_shared = True
                vm_disk.read_write = 'w'
                isMatched=True
                #calculate disk size here
                op = "+"
                self.storage_manager.calculate_disk_size(storage_disk.id, vm_disk.id, vm_disk.disk_size, op)
            elif vm_disk.read_write == "r":
                #storage is allocated
                LOGGER.info("Storage (" + to_str(vm_disk.disk_name) + ") is allocated. Adding vm disk link in readonly mode.")
                self.add_vm_storage_link(vm_disk.id, storage_disk.id)
                #update the shared flag
                vm_disk.is_shared = True
                isMatched=True
                #calculate disk size here
                op = "+"
                self.storage_manager.calculate_disk_size(storage_disk.id, vm_disk.id, vm_disk.disk_size, op)
            else:
                isMatched=True
                vm_link = DBSession.query(VMStorageLinks).filter_by(vm_disk_id=vm_disk.id, storage_disk_id=storage_disk.id).first()
                if vm_link:
                    LOGGER.info("vm disk and storage disk link already exists so we are not throwing exception or logging error.")
                else:
                    LOGGER.error("Matching disk (" + str(vm_disk.disk_name) + ") is found. The disk is already allocated and in read-write mode. So it is invalid disk entry.")
                    msg = "Invalid disk entry."
                    
        return (isMatched, msg)
                