#!/usr/bin/env python
#
#   ConVirt   -  Copyright (c) 2008 Convirture Corp.
#   ======
#
# ConVirt is a Virtualization management tool with a graphical user
# interface that allows for performing the standard set of VM operations
# (start, stop, pause, kill, shutdown, reboot, snapshot, etc...). It
# also attempts to simplify various aspects of VM lifecycle management.
#
#
# This software is subject to the GNU General Public License, Version 2 (GPLv2)
# and for details, please consult it at:
#
#    http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt
# 
#
# author : ConVirt Team
#
# -*- coding: utf-8 -*-

from sqlalchemy.orm import mapper, relation
from sqlalchemy import Table, ForeignKey, Column
from sqlalchemy.types import Integer, Unicode, String, Boolean, PickleType, Float, DateTime
from sqlalchemy.orm import relation, backref

from convirt.model import DeclarativeBase, metadata, DBSession
from convirt.model.Groups import ServerGroup
from convirt.model.Sites import Site
from convirt.model.ManagedNode import ManagedNode

from sqlalchemy.schema import UniqueConstraint,Index
from convirt.core.utils.utils import uuidToString, randomUUID, dynamic_map

class Storage_Stats(DeclarativeBase):
    __tablename__ = 'storage_stats'

    #Columns
    id = Column(Integer, autoincrement=True, primary_key=True)
    storage_id = Column(Unicode(50), ForeignKey('storage_definitions.id'))
    total_size = Column(Float)
    used = Column(Float)

    #Relation
    str_def = relation('StorageDef', backref='Storage_Stats')

    def __init__(self):pass

Index("strgestat_id", Storage_Stats.id)

#Would establish Server and Storage/Network Definition link
class ServerDefLink(DeclarativeBase):
    __tablename__ = 'serverdeflinks'
    
    #Composite unique constraint
    __table_args__ = (UniqueConstraint('server_id', 'def_id', name='ucServerDefLink'), {})

    #Columns
    id = Column(Integer, autoincrement=True, primary_key=True)
    server_id = Column(Unicode(50), ForeignKey('managed_nodes.id'))    #From ManagedNode
    def_type = Column(Unicode(50))
    def_id = Column(Unicode(50))
    status = Column(Unicode(50))
    details = Column(Unicode(500))
    dt_time = Column(DateTime)

    #Relation
    mng_node = relation('ManagedNode', backref='ServerDefLink')

    def __init__(self):pass

Index("sdef_sid_dtype_did", ServerDefLink.server_id,ServerDefLink.def_type,ServerDefLink.def_id)

#Would establish ServerPool and Definition link
class SPDefLink(DeclarativeBase):
    __tablename__ = 'spdeflinks'

    #Composite unique constraint
    __table_args__ = (UniqueConstraint('group_id', 'def_id', name='ucSPDefLink'), {})

    #Columns
    id = Column(Integer, autoincrement=True, primary_key=True)
    group_id = Column(Unicode(50), ForeignKey('server_groups.id'))    #From ServerGroup
    def_type = Column(Unicode(50))
    def_id = Column(Unicode(50))# this would be the def id either from Storage definition or from network definition
    status = Column(Unicode(50)) #not sure of what status
    oos_count = Column(Integer) #would display the count of servers which are out of Sync
    dt_time = Column(DateTime)

    #Relation
    svr_grp = relation('ServerGroup', backref='SPDefLink')

    def __init__(self):pass

Index("spdef_gid_dtype_did", SPDefLink.group_id,SPDefLink.def_type,SPDefLink.def_id)

#Would establish ServerPool and Definition link
class DCDefLink(DeclarativeBase):
    __tablename__ = 'dcdeflinks'

    #Composite unique constraint
    __table_args__ = (UniqueConstraint('site_id', 'def_id', name='ucDCDefLink'), {})

    #Columns
    id = Column(Integer, autoincrement=True, primary_key=True)
    site_id = Column(Unicode(50), ForeignKey('sites.id'))    #From Site
    def_type = Column(Unicode(50))
    def_id = Column(Unicode(50))# this would be the def id either from Storage definition or from network definition
    status = Column(Unicode(50)) #not sure of what status
    oos_count = Column(Integer) #would display the count of servers which are out of Sync
    dt_time = Column(DateTime)

    #Relation
    site_grp = relation('Site', backref='DCDefLink')

    def __init__(self):pass

Index("dcdef_sid_dtype_did", DCDefLink.site_id,DCDefLink.def_type,DCDefLink.def_id)

class StorageDisks(DeclarativeBase):
    __tablename__ = 'storage_disks'

    #Columns
    id = Column(Unicode(50), primary_key=True)
    storage_id = Column(Unicode(50), ForeignKey('storage_definitions.id'))
    storage_type = Column(Unicode(50))
    disk_name = Column(Unicode(100))
    mount_point = Column(Unicode(50))
    file_system = Column(Unicode(50))
    actual_size = Column(Float)
    size = Column(Float)
    unique_path = Column(Unicode(100))
    current_portal = Column(Unicode(50))
    target = Column(Unicode(50))
    state = Column(Unicode(50))
    lun = Column(Integer)
    storage_allocated = Column(Boolean)

    #Relation
    fk_StorageDisks_StorageDef = relation('StorageDef', backref='StorageDisks')
    
    #Index
    #create index StorageDisks_uniquePath on StorageDisks(unique_path)
    #create index StorageDisks_diskName on StorageDisks(disk_name)

    def __init__(self):pass
