# EmailService.py
#
#   This module contains code to send email to the requested user
#
from convirt.model.DBHelper import DBHelper
from convirt.core.utils.utils import constants,getHexID
from convirt.core.utils.utils import to_unicode,to_str
import convirt.core.utils.constants
constants = convirt.core.utils.constants
import traceback
from convirt.model import DBSession
from convirt.model.EmailSetup import EmailSetup
from convirt.model.Sites import Site
from datetime import datetime, date
import smtplib
from convirt.core.utils import ssmtplib
from email.MIMEText import MIMEText
from tg import session
from convirt.model.auth import User
import time
from convirt.model.Authorization import AuthorizationService
from convirt.model.Credential import Credential
from convirt.model.Entity import Entity, EntityType
import transaction


NONSECURE= 1
TLS = 2
SSL = 3

def ct_time():
    return datetime.utcnow().ctime()

class EmailService:
    def __init__(self):
        self.sender = None
        self.receivers = None
        self.message = "Test Message"
        self.mail_server = None
        self.port = None
        self.description = None
        self.secure_type = None
        self.text_subtype = 'plain'
        self.content="\Test message Send on " + to_str(ct_time())
        self.msg = MIMEText(self.content, self.text_subtype)
        self.subject = 'Test'
        self.msg['Subject']= self.subject
        self.auth=AuthorizationService()
        
    def add_entity(self,name,entity_id, entityType ,parent):
        try:
            type=DBHelper().find_by_name(EntityType,entityType)
            e=Entity()
            e.name=name
            e.type=type
            e.entity_id=entity_id
            e.parent=parent
            DBHelper().add(e)
        except Exception, e:
            raise e

        return e

    # API to save EmailSetup dialog entries into the EmailSetup/Credentials table
    def save_email_setup_details(self, desc, servername, port, useremail, password, secure):
        SiteRecord = DBSession.query(Site).filter(Site.name == 'Data Center').first()
        if SiteRecord:
            site_id = SiteRecord.id
            # check if record for same server name avoid duplicate records of same name
            EmailRecord =  DBSession.query(EmailSetup).filter(EmailSetup.site_id == site_id).filter(EmailSetup.mail_server == servername).first()
            if EmailRecord:
                return dict(success=True,msg="Duplicaate Record found in list")
 
            else:
                # Add record in EmailSetup table for site id queried
                email_setup_obj = EmailSetup(servername, desc, port, secure, site_id, useremail, password )
                DBSession.add(email_setup_obj)
                emailsetupid = email_setup_obj.getEmailSetupId()
                self.add_entity(to_unicode(servername), emailsetupid, to_unicode(constants.EMAIL), None)
                return dict(success=True, msg="New Record Added Sucessfully")
 
    # API to update EmailSetup dialog entries into the EmailSetup/Credentials table
    def update_email_setup_details(self, desc, servername, port, useremail, password, secure):
        emaildetails = DBSession.query(EmailSetup).filter(EmailSetup.mail_server== servername).first()
        if emaildetails:
            emaildetails.description = desc
            emaildetails.port = port
            emaildetails.use_secure = secure
            emaildetails.credential.cred_details['user_email'] = useremail
            emaildetails.credential.cred_details['password'] = password
            return dict(success=True, msg="Record updated sucessfully")
        else:
            return dict(success=False, msg="Record updation failed")

    # Apis invoked from Email Setup dialog to send test mail with entries made
    # in the dialog. 
    def send_test_email(self, desc, servername, port, useremail, password, secure):
        self.sender = useremail       
        Record = DBSession.query(User.email_address).filter(User.user_name =='admin').first()
        self.receivers =Record.email_address
        
        self.mail_server = servername
        if port:
            self.port = int(port)
        self.secure_type = int(secure)
        self.password = password
        self.subject = "Test Email"
        self.content="\Test message Sent on " + to_str(ct_time())
        self.msg = MIMEText(self.content, self.text_subtype)
 
        SendSuccess = False
        if (self.secure_type== NONSECURE):
            SendSuccess= self.send_nonsecure()
        elif (self.secure_type== TLS):
            SendSuccess = self.send_tls()
        else:
            SendSuccess= self.send_ssl()

        if (SendSuccess== True):
            message = "mail send to " + Record.email_address
            return message
        else:
            message = "Test Failed"
            return message

    # Test all server connection by sending the mail
    def get_mailservers(self):  
        MailSetupRecord = []
        SiteRecord = DBSession.query(Site).filter(Site.name == 'Data Center').first()
        if SiteRecord:
            site_id = SiteRecord.id
        emailsetuprecords = DBHelper().filterby(EmailSetup,[], [EmailSetup.site_id == site_id])
        for emailSetupobj in emailsetuprecords:            
            MailSetupRecord.append(dict(MailSetup=emailSetupobj, Creds= emailSetupobj.credential))
        return MailSetupRecord

    def send_email_to_client(self, msg, receiver):
        # Query sender and password from email credential table
        # Query mail_server,port,use_secure from the email setup table for curenly  logged in user
        self.receivers = receiver
        self.content=msg

        emailservers = self.get_mailservers()
        for eachmailserver in emailservers:
            if eachmailserver:
                self.mail_server = eachmailserver['MailSetup'].mail_server
                self.port = int(eachmailserver['MailSetup'].port)
                self.secure_type = int(eachmailserver['MailSetup'].use_secure)
                self.cred_details = eachmailserver['Creds'].cred_details
                self.password = self.cred_details['password']
                self.sender = self.cred_details['user_email'] 
                result = False

                if (self.secure_type== NONSECURE):
                    result = self.send_nonsecure()
                elif (self.secure_type== TLS):
                    result = self.send_tls()
                else:
                    result = self.send_ssl()

                if (result == True):
                    return "Test mail sent from " + eachmailserver['MailSetup'].mail_server

    def send_email_to_user(self, msg):
        # Query sender and password from email credential table
        # Query mail_server,port,use_secure from the email setup table for curenly  logged in user 
        # receiver: to be queried from users table 
        self.msg = msg
        curr_user_id = session.get('userid')
        #query users table to retrieve email address of currenlt logged in user
        userRecord = DBSession.query(User.email_address).filter(User.user_name == curr_user_id).first()
        if userRecord:
            self.receivers = userRecord.email_address
        emailservers = self.get_mailservers()
        for eachmailserver in emailservers:
            if eachmailserver:
                self.mail_server = eachmailserver['MailSetup'].mail_server
                self.port = int(eachmailserver['MailSetup'].port)
                self.secure_type = int(eachmailserver['MailSetup'].use_secure)
                self.cred_details = eachmailserver['Creds'].cred_details
                self.password = self.cred_details['password']
                self.sender = self.cred_details['user_email']
                result = False
                if (self.secure_type== NONSECURE):
                    result = self.send_nonsecure()
                elif (self.secure_type== TLS):
                    result = self.send_tls()
                else:
                    result = self.send_ssl()

                if (result == True):
                    return "Test mail sent from " + eachmailserver['MailSetup'].mail_server

    # to send email to non secure channel
    def send_nonsecure(self):
        try:
            smtpObj = smtplib.SMTP(self.mail_server, self.port)
            #server.login(self.sender, self.password)
            smtpObj.sendmail(self.sender, self.receivers, self.msg.as_string())
            return True
        except:
            return False

    # to send email using TLS secure channel
    def send_tls(self):
        try:
            server = smtplib.SMTP(self.mail_server, self.port)
            server.set_debuglevel(False)
            server.ehlo()
            server.starttls()
            server.ehlo()
            server.login(self.sender, self.password)
            server.sendmail(self.sender, self.receivers, self.msg.as_string())
            server.close()
            return True
        except:
            return False

    # to send email using SSL secure chanel
    def send_ssl(self):
        try:
            server = ssmtplib.SMTP_SSL(self.mail_server, self.port)
            server.login(self.sender, self.password)
            try:
                server.sendmail(self.sender, self.receivers, self.msg.as_string())
            finally:
                server.close()
                return True
        except Exception, exc:
            return False

    def get_emailsetupinfo(self):
        result= []
        emailsetuptype=DBHelper().get_all(EmailSetup) #site id needs to be added in the filter
        for emailSetupobj in emailsetuptype:
            result.append(dict(emailsetup_id=emailSetupobj.id,servername=emailSetupobj.mail_server, desc=emailSetupobj.description, username = emailSetupobj.credential.cred_details['user_email'], port = emailSetupobj.port) ) 
        return result

    def delete_emailrecord(self, emailsetup_id):
        emailSetupobj=DBHelper().find_by_id(EmailSetup,emailsetup_id)
        DBHelper().delete(emailSetupobj)
        entity_obj= DBSession.query(Entity).filter(Entity.entity_id == emailsetup_id).first()
        DBHelper().delete(entity_obj)

    def get_emailsetup_details(self,emailsetup_id):
        result=[]
        emailSetupobj=DBHelper().find_by_id(EmailSetup,emailsetup_id) 
        result.append(dict(emailsetup_id=emailSetupobj.id, servername=emailSetupobj.mail_server, desc=emailSetupobj.description, port = emailSetupobj.port, use_secure = emailSetupobj.use_secure ,username = emailSetupobj.credential.cred_details['user_email'], password = emailSetupobj.credential.cred_details['password']))
        return result

